==> heptapod-rails-0.36.1.tar.gz: heptapod-rails-0.36.1/spec/models/project_hg_native_spec.rb <==
# frozen_string_literal: true

require 'spec_helper'

RSpec.describe 'Project and Repository models with vcs_type="hg"', factory_default: :keep do
  let(:project) do
    allow_next_instance_of(Project) do |project|
      allow(project).to receive(:check_vcs_type).and_return(true)
    end
    create(:native_hg_project)
  end

  let(:raw_repo) { project.repository.raw_repository }
  let(:raw_wiki_repo) { project.wiki.repository.raw_repository }

  describe '#hg_with_git?' do
    context 'when hg_without_git feature flag is instance-wide enabled' do
      before do
        stub_feature_flags(hg_without_git: true)
      end

      it 'is false for main repo but not for wiki if fallback is not enabled' do
        stub_feature_flags(hg_with_git_fallback: false)
        expect(raw_repo).to be_instance_of(Gitlab::Mercurial::HgitalyRepository)
        expect(raw_repo.hg_with_git?).to be(false)
        expect(raw_wiki_repo.hg_with_git?).to be(true)
      end

      it 'is true if fallback is enabled' do
        stub_feature_flags(hg_with_git_fallback: true)
        expect(raw_repo).to be_instance_of(Gitlab::Mercurial::HgitalyRepository)
        expect(raw_repo.hg_with_git?).to be(true)
        expect(raw_wiki_repo.hg_with_git?).to be(true)
      end
    end

    context 'when hg_without_git feature flag is instance-wide disabled' do
      before do
        stub_feature_flags(hg_without_git: false)
      end

      it 'is true' do
        expect(raw_repo).to be_instance_of(Gitlab::Mercurial::HgitalyRepository)
        expect(raw_repo.hg_with_git?).to be(true)
        expect(raw_wiki_repo.hg_with_git?).to be(true)
      end
    end
  end
end

==> heptapod-rails-0.36.2.tar.gz: heptapod-rails-0.36.2/spec/models/project_hg_native_spec.rb <==
# frozen_string_literal: true

require 'spec_helper'

RSpec.describe 'Project and Repository models with vcs_type="hg"', factory_default: :keep do
  let(:project) do
    allow_next_instance_of(Project) do |project|
      allow(project).to receive(:check_vcs_type).and_return(true)
    end
    create(:native_hg_project)
  end

  let(:raw_repo) { project.repository.raw_repository }
  let(:raw_wiki_repo) { project.wiki.repository.raw_repository }

  describe '#hg_legacy_git_conversion' do
    it 'is false on newly created projects' do
      expect(project.hg_legacy_git_conversion).to be(false)
    end

    it 'is nil on previously created projects' do
      # forcing to nil in the database, then rereading
      project.hg_legacy_git_conversion = nil
      project.save!

      reread = Project.find(project.id)
      expect(reread.equal?(project)).to be(false) # checking test assumption: we got a new object
      expect(reread.hg_legacy_git_conversion).to be_nil # default not applied to existing record
    end
  end

  describe '#hg_with_git?' do
    context 'when hg_without_git feature flag is instance-wide enabled' do
      before do
        stub_feature_flags(hg_without_git: true)
      end

      it 'is false for main repo but not for wiki if fallback is not enabled and setting is nil' do
        stub_feature_flags(hg_with_git_fallback: false)
        project.hg_legacy_git_conversion = nil
        expect(raw_repo).to be_instance_of(Gitlab::Mercurial::HgitalyRepository)
        expect(raw_repo.hg_with_git?).to be(false)
        expect(raw_wiki_repo.hg_with_git?).to be(true)
      end

      it 'is true if fallback is enabled and setting is nil' do
        stub_feature_flags(hg_with_git_fallback: true)
        project.hg_legacy_git_conversion = nil
        expect(raw_repo).to be_instance_of(Gitlab::Mercurial::HgitalyRepository)
        expect(raw_repo.hg_with_git?).to be(true)
        expect(raw_wiki_repo.hg_with_git?).to be(true)
      end

      it 'is true if fallback is not enabled and setting is true' do
        stub_feature_flags(hg_with_git_fallback: false)
        project.hg_legacy_git_conversion = true
        expect(raw_repo).to be_instance_of(Gitlab::Mercurial::HgitalyRepository)
        expect(raw_repo.hg_with_git?).to be(true)
        expect(raw_wiki_repo.hg_with_git?).to be(true)
      end

      it 'is true if fallback is enabled and setting is false' do
        stub_feature_flags(hg_with_git_fallback: true)
        project.hg_legacy_git_conversion = false
        expect(raw_repo).to be_instance_of(Gitlab::Mercurial::HgitalyRepository)
        expect(raw_repo.hg_with_git?).to be(true)
        expect(raw_wiki_repo.hg_with_git?).to be(true)
      end

      it 'is false if fallback is not enabled and setting is false' do
        stub_feature_flags(hg_with_git_fallback: false)
        project.hg_legacy_git_conversion = false
        expect(raw_repo).to be_instance_of(Gitlab::Mercurial::HgitalyRepository)
        expect(raw_repo.hg_with_git?).to be(false)
        expect(raw_wiki_repo.hg_with_git?).to be(true)
      end
    end

    context 'when hg_without_git feature flag is instance-wide disabled' do
      before do
        stub_feature_flags(hg_without_git: false)
      end

      it 'is true if setting is nil' do
        project.hg_legacy_git_conversion = nil
        expect(raw_repo).to be_instance_of(Gitlab::Mercurial::HgitalyRepository)
        expect(raw_repo.hg_with_git?).to be(true)
        expect(raw_wiki_repo.hg_with_git?).to be(true)
      end

      it 'is true if setting is false' do
        project.hg_legacy_git_conversion = false
        expect(raw_repo).to be_instance_of(Gitlab::Mercurial::HgitalyRepository)
        expect(raw_repo.hg_with_git?).to be(true)
        expect(raw_wiki_repo.hg_with_git?).to be(true)
      end

      it 'is true if setting is true' do
        project.hg_legacy_git_conversion = true
        expect(raw_repo).to be_instance_of(Gitlab::Mercurial::HgitalyRepository)
        expect(raw_repo.hg_with_git?).to be(true)
        expect(raw_wiki_repo.hg_with_git?).to be(true)
      end
    end
  end
end
